{
    Scheme Maker is an application designed to enable you
    to create knitting and cross-stitch schemes from images and photos.
    Created 2010-07-05
    Last modified 2010-08-12
    Current version 1.3
    Copyright (C) 2010  Mariya Fomkina <mariyafomkina@users.sourceforge.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit Main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, Menus, ExtDlgs, Spin, StdCtrls, ComCtrls, XMLPropStorage,
  FastBitmapRoutines, DefaultTranslator, LazHelpHTML, About, HelpIntfs, LCLIntf,
  gettext, LCLType, IntfGraphics, fpImage, Mouline, PaperPreviewUnit;

type

  TSchemeCondition =
    (scNone,
     scOpened,
     scLoaded,
     scDetected,
     scLessened);

  { TMainForm }

  TMainForm = class(TForm)
    bReturn: TButton;
    bDetect: TButton;
    bLessen: TButton;
    fseCm: TFloatSpinEdit;
    fseInch: TFloatSpinEdit;
    gbCanvas: TGroupBox;
    iImage: TImage;
    iSelection: TImage;
    iScheme: TImage;
    lC: TLabel;
    lSize1: TLabel;
    lSize2: TLabel;
    lW: TLabel;
    lH: TLabel;
    lP: TLabel;
    OpenPictureDialog1: TOpenPictureDialog;
    pBase: TPanel;
    rgSize: TRadioGroup;
    SavePictureDialog1: TSavePictureDialog;
    sbImage: TScrollBox;
    MainMenu1: TMainMenu;
    miSave: TMenuItem;
    miFile: TMenuItem;
    miOpen: TMenuItem;
    sbScheme: TScrollBox;
    seC: TSpinEdit;
    seW: TSpinEdit;
    seH: TSpinEdit;
    seP: TSpinEdit;
    rgDetection: TRadioGroup;
    bLoad: TButton;
    StatusBar1: TStatusBar;
    ProgressBar1: TProgressBar;
    pOptions: TPanel;
    Splitter1: TSplitter;
    lArrow12: TLabel;
    lArrow23: TLabel;
    lArrow34: TLabel;
    cbProportion: TCheckBox;
    XMLPropStorage1: TXMLPropStorage;
    miHelp: TMenuItem;
    miManual: TMenuItem;
    miSep2: TMenuItem;
    miAbout: TMenuItem;
    ToolBar1: TToolBar;
    tbOpen: TToolButton;
    tbSave: TToolButton;
    tbSep: TToolButton;
    tbManual: TToolButton;
    tbExit: TToolButton;
    miSep1: TMenuItem;
    miExit: TMenuItem;
    ImageList1: TImageList;
    HTMLHelpDatabase1: THTMLHelpDatabase;
    HTMLBrowserHelpViewer1: THTMLBrowserHelpViewer;
    miHome: TMenuItem;
    procedure iImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure iImageMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer
      );
    procedure iImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ProjectImage(Sender: TObject);
    procedure ChangeCanvas(Sender: TObject);
    procedure fseCmChange(Sender: TObject);
    procedure fseInchChange(Sender: TObject);
    procedure rgSizeClick(Sender: TObject);
    procedure FirstStep(Sender: TObject);
    procedure SecondStep(Sender: TObject);
    procedure ThirdStep(Sender: TObject);
    procedure ForthStep(Sender: TObject);
    procedure ActiveButton(Sender: TObject; button: TButton);
    procedure InactiveButton(Sender: TObject; button: TButton);
    procedure ChangeEnablility(Sender: TObject; b2, b3, b4: boolean; k: byte);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure miOpenClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure miManualClick(Sender: TObject);
    procedure miHomeClick(Sender: TObject);
    procedure miAboutClick(Sender: TObject);
    procedure seWChange(Sender: TObject);
    procedure seHChange(Sender: TObject);
    procedure sePChange(Sender: TObject);
    procedure seCChange(Sender: TObject);
    procedure bLoadClick(Sender: TObject);
    procedure bDetectClick(Sender: TObject);
    procedure bLessenClick(Sender: TObject);
    procedure bReturnClick(Sender: TObject);
    procedure cgGrid_ItemClick(Sender: TObject; Index: integer);
  private
    { private declarations }
  public
    SchCond: TSchemeCondition;
    sW, sH, sP, sC: integer;
    OriginalColors, LessenedColors: ColorAmountArray;
    OrigClNumber, LessClNumber: integer;
    iTemp, iTempForCancel: TImage;
    IsSaved: boolean;
    { public declarations }
  end;

  TColorInfo = record
    color: TColor;
    number: integer;
  end;

var
  MainForm: TMainForm;
  bmTemp: TBitMap;
  ColorList: TList;
  HelpPath, HomePath: string;
  SelectedRect: TRect;
  iStart: TImage;
  MouseIsDown: Bool;

resourcestring
  Hint1 = 'Select File|Open... to select an image for scheme';
  Hint2 = 'Press "1. Load" to create the base for scheme';
  Hint3 = 'Press "2. Detect" to detect color for each cell of the scheme';
  Hint4 = 'Press "3. Lessen" to lessen the number of colors to the "Final colors"';
  Hint5 = 'Press "4. Return" to return the original number of colors in the scheme';
  Hint6 = 'Select File|Save... to save scheme or repeat any step before';
  cdItem0 = 'Central point';
  cdItem1 = 'Average (corners)';
  cdItem2 = 'Average (whole cell)';
  rgsItem0 = 'cm';
  rgsItem1 = 'inch';
  rgsItem2 = 'cells';
  QCaption = 'Scheme Maker';
  Question1 = 'All the changes in the scheme will be lost. ' +
              'Do you want to continue?';
  Question2 = 'The scheme was changed. Do you want to save it?';

implementation

{ TMainForm }

//The projection of the image in temp bitmap to scheme image
procedure TMainForm.ProjectImage(Sender: TObject);
begin
  CopyBitmapToBitmapWithScale(iTemp.Picture.Bitmap, iScheme.Picture.Bitmap,
                              sW * sP / iTemp.Picture.Bitmap.Width,
                              sH * sP / iTemp.Picture.Bitmap.Height);
  DrawGridOnBitmap(iScheme.Picture.Bitmap, sP, sP, 1, RGBToColor(0, 0, 1));
  DrawGridOnBitmap(iScheme.Picture.Bitmap, sP * 10, sP * 10, 2, RGBToColor(0, 0, 1));
end;

procedure DrawSelectedRect(var im: TImage; var r: TRect);
begin
  with im.Canvas do
    begin
      Pen.Color := clBlue;
      Pen.Width := 1;
      MoveTo(r.Left, r.Top);
      LineTo(r.Right - 1, r.Top);
      LineTo(r.Right - 1, r.Bottom - 1);
      LineTo(r.Left, r.Bottom - 1);
      LineTo(r.Left, r.Top);
      Line(r.Left + 1, r.Top + 1, r.Left + 5, r.Top + 5);
      Line(r.Right - 1, r.Top + 1, r.Right - 5, r.Top + 5);
      Line(r.Right - 1, r.Bottom - 1, r.Right - 5, r.Bottom - 5);
      Line(r.Left + 1, r.Bottom - 1, r.Left + 5, r.Bottom - 5);
    end;
end;

//Used when the user selects the rect to make scheme from
procedure TMainForm.iImageMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if MouseIsDown then
    begin
      CopyBitmapToBitmap(iStart.Picture.Bitmap, iImage.Picture.Bitmap);

      if Y < 0 then
        SelectedRect.Bottom := 0
      else if Y > iImage.Picture.Bitmap.Height then
        SelectedRect.Bottom := iImage.Picture.Bitmap.Height
      else
        SelectedRect.Bottom := Y;

      if X < 0 then
        SelectedRect.Right := 0
      else if X > iImage.Picture.Bitmap.Width then
        SelectedRect.Right := iImage.Picture.Bitmap.Width
      else
        SelectedRect.Right := X;

      DrawSelectedRect(iImage, SelectedRect);
      sW := abs(SelectedRect.Right - SelectedRect.Left);
      sH := abs(SelectedRect.Bottom - SelectedRect.Top);
      rgSizeClick(Self);
    end;
end;

procedure TMainForm.iImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  t: integer;
begin
  MouseIsDown := False;
  if SelectedRect.Bottom < SelectedRect.Top then
    begin
      t := SelectedRect.Bottom;
      SelectedRect.Bottom := SelectedRect.Top;
      SelectedRect.Top := t;
    end;
  if SelectedRect.Right < SelectedRect.Left then
    begin
      t := SelectedRect.Right;
      SelectedRect.Right := SelectedRect.Left;
      SelectedRect.Left := t;
    end;
end;

procedure TMainForm.iImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseIsDown := True;
  SelectedRect.Top := Y;
  SelectedRect.Left := X;
end;

//Used when canvas propertities are changed
procedure TMainForm.ChangeCanvas(Sender: TObject);
begin
  case rgSize.ItemIndex of
    0:
      begin
        sW := Round (seW.Value * fseCm.Value);
        sH := Round (seH.Value * fseCm.Value);
        FirstStep(Self);
      end;
    1:
      begin
        sW := Round (seW.Value * fseInch.Value);
        sH := Round (seH.Value * fseInch.Value);
        FirstStep(Self);
      end;
  end;
end;

procedure TMainForm.fseCmChange(Sender: TObject);
begin
  fseInch.Value := fseCm.Value * 2.54;
  ChangeCanvas (Self);
end;

procedure TMainForm.fseInchChange(Sender: TObject);
begin
  fseCm.Value := fseInch.Value / 2.54;
  ChangeCanvas (Self);
end;

procedure TMainForm.rgSizeClick(Sender: TObject);
begin
  case rgSize.ItemIndex of
    0:
      begin
        seW.Value := Round (sW / fseCm.Value);
        seH.Value := Round (sH / fseCm.Value);
      end;
    1:
      begin
        seW.Value := Round (sW / fseInch.Value);
        seH.Value := Round (sH / fseInch.Value);
      end;
    2:
      begin
        seW.Value := sW;
        seH.Value := sH;
      end;
  end;
end;

//Load (the image from source to the scheme image and to the temp bitmap)
procedure TMainForm.FirstStep(Sender: TObject);
begin
  iScheme.Width := sW * sP;
  iScheme.Height := sH * sP;
  iScheme.Picture.Bitmap.Width := iScheme.Width;
  iScheme.Picture.Bitmap.Height := iScheme.Height;
  ProjectImage(Self);
  CopyBitmapToBitmapWithScale(iTemp.Picture.Bitmap, bmTemp,
                              sW * sP / iTemp.Picture.Bitmap.Width,
                              sH * sP / iTemp.Picture.Bitmap.Height);
end;

//Detect and Analize (detect color for every cell of the scheme)
procedure TMainForm.SecondStep(Sender: TObject);
var
  SrcIntfImg, TempIntfImg: TLazIntfImage;
  ImgHandle, ImgMaskHandle: HBitmap;
  i, j, l, k: integer;
  r, g, b: integer;
  del: integer;
begin
  for i := 1 to MoulineAmount do
    OriginalColors[i] := 0;

  iTemp.Picture.Bitmap.Width := sW;
  iTemp.Picture.Bitmap.Height := sH;

  SrcIntfImg := TLazIntfImage.Create(0, 0);
  SrcIntfImg.LoadFromBitmap(bmTemp.Handle, bmTemp.MaskHandle);
  TempIntfImg := TLazIntfImage.Create(0,0);
  TempIntfImg.DataDescription := SrcIntfImg.DataDescription;
  TempIntfImg.SetSize(sW, sH);

  ProgressBar1.Max := sW;
  ProgressBar1.Position := 0;
  ProgressBar1.Visible := True;

  case rgDetection.ItemIndex of
    0:
      begin
        del := (sP + 1) div 2;
        for j := 0 to sH - 1 do
          begin
            for i := 0 to sW - 1 do
              begin
                TempIntfImg.Colors[i, j] := GetNearestColor(
                  SrcIntfImg.Colors[i * sP + del, j * sP + del]);
                inc(OriginalColors[GetNearestColorIndex(
                  TempIntfImg.Colors[i, j])]);
              end;
            ProgressBar1.StepIt;
          end;
      end;
    1:
      for j := 0 to sH - 1 do
        begin
          for i := 0 to sW - 1 do
            begin
              r := Round((
                SrcIntfImg.Colors[i * sP, j * sP].red +
                SrcIntfImg.Colors[i * sP + sP - 1, j * sP].red +
                SrcIntfImg.Colors[i * sP, j * sP + sP - 1].red +
                SrcIntfImg.Colors[i * sP + sP - 1, j * sP + sP - 1].red) / 4);
              g := Round((
                SrcIntfImg.Colors[i * sP, j * sP].green +
                SrcIntfImg.Colors[i * sP + sP - 1, j * sP].green +
                SrcIntfImg.Colors[i * sP, j * sP + sP - 1].green +
                SrcIntfImg.Colors[i * sP + sP - 1, j * sP + sP - 1].green) / 4);
              b := Round((
                SrcIntfImg.Colors[i * sP, j * sP].blue +
                SrcIntfImg.Colors[i * sP + sP - 1, j * sP].blue +
                SrcIntfImg.Colors[i * sP, j * sP + sP - 1].blue +
                SrcIntfImg.Colors[i * sP + sP - 1, j * sP + sP - 1].blue) / 4);
              TempIntfImg.Colors[i, j] := GetNearestColor(FPColor(r, g, b));
              inc(OriginalColors[GetNearestColorIndex(
                TempIntfImg.Colors[i, j])]);
            end;
          ProgressBar1.StepIt;
        end;
    2:
      for j := 0 to sH - 1 do
        begin
          for i := 0 to sW - 1 do
            begin
              r := 0;
              g := 0;
              b := 0;
              for l := 0 to sP - 1 do
                for k := 0 to sP - 1 do
                  begin
                    r := r + SrcIntfImg.Colors[i * sP + l, j * sP + k].red;
                    g := g + SrcIntfImg.Colors[i * sP + l, j * sP + k].green;
                    b := b + SrcIntfImg.Colors[i * sP + l, j * sP + k].blue;
                  end;
              r := Round(r / sP / sP);
              g := Round(g / sP / sP);
              b := Round(b / sP / sP);
              TempIntfImg.Colors[i, j] := GetNearestColor(FPColor(r, g, b));
              inc(OriginalColors[GetNearestColorIndex(
                TempIntfImg.Colors[i, j])]);
            end;
          ProgressBar1.StepIt;
        end;
  end;

  TempIntfImg.CreateBitmaps(ImgHandle, ImgMaskHandle, false);
  iTemp.Picture.Bitmap.Handle := ImgHandle;
  iTemp.Picture.Bitmap.MaskHandle := ImgMaskHandle;
  SrcIntfImg.Destroy;
  TempIntfImg.Destroy;

  OrigClNumber := 0;
  for i := 1 to MoulineAmount do
    if OriginalColors[i] > 0 then
      inc(OrigClNumber);
  seC.MaxValue := OrigClNumber;
  seC.Value := OrigClNumber;
  sC := OrigClNumber;

  CopyBitmapToBitmap(iTemp.Picture.Bitmap, iTempForCancel.Picture.Bitmap);
  ProjectImage(Self);
  ProgressBar1.Visible := False;
end;

//Lessen (the number of colors to the nessesary)
procedure TMainForm.ThirdStep(Sender: TObject);
var
  SrcIntfImg: TLazIntfImage;
  ImgHandle, ImgMaskHandle: HBitmap;
  i, j, k: integer;
  changedindex, changingindex: integer;
  col1, col2: TFPColor;
  cl1, cl2: TColor;
  dif, dif2, num: integer;
begin
  SrcIntfImg := TLazIntfImage.Create(0, 0);
  SrcIntfImg.LoadFromBitmap(iTemp.Picture.Bitmap.Handle,
    iTemp.Picture.Bitmap.MaskHandle);

  ProgressBar1.Max := OrigClNumber - sC;
  ProgressBar1.Position := 0;
  ProgressBar1.Visible := True;

  LessClNumber := OrigClNumber;
  for i := 1 to MoulineAmount do
    LessenedColors[i] := OriginalColors[i];

  while LessClNumber > sC do
    begin
      changedindex := 1;
      while LessenedColors[changedindex] = 0 do
        inc(changedindex);
      for k := changedindex + 1 to MoulineAmount do
        if LessenedColors[k] <> 0 then
          if LessenedColors[k] < LessenedColors[changedindex] then
            changedindex := k;
      num := LessenedColors[changedindex];
      LessenedColors[changedindex] := 0;
      cl1 := ColorTable[changedindex];

      changingindex := 1;
      while LessenedColors[changingindex] = 0 do
        inc(changingindex);
      cl2 := ColorTable[changingindex];
      dif := sqr(Red(cl2) - Red(cl1)) + sqr(Green(cl2) - Green(cl1)) +
        sqr(Blue(cl2) - Blue(cl1));

      for k := changingindex + 1 to MoulineAmount do
        if LessenedColors[k] <> 0 then
          begin
            cl2 := ColorTable[k];
            dif2 := sqr(Red(cl2) - Red(cl1)) + sqr(Green(cl2) - Green(cl1)) +
              sqr(Blue(cl2) - Blue(cl1));
            if dif2 < dif then
              begin
                changingindex := k;
                dif := dif2;
              end;
          end;
      inc(LessenedColors[changingindex], num);
      cl2 := ColorTable[changingindex];

      col1 := TColorToFPColor(cl1);
      col2 := TColorToFPColor(cl2);

      for j := 0 to sH - 1 do
        for i := 0 to sW - 1 do
          if SrcIntfImg.Colors[i, j] = col1 then
            SrcIntfImg.Colors[i, j] := col2;

      dec(LessClNumber);
      ProgressBar1.StepIt;
    end;

  SrcIntfImg.CreateBitmaps(ImgHandle, ImgMaskHandle, false);
  iTemp.Picture.Bitmap.Handle := ImgHandle;
  iTemp.Picture.Bitmap.MaskHandle := ImgMaskHandle;
  SrcIntfImg.Destroy;

  ProjectImage(Self);
  ProgressBar1.Visible := False;
end;

//Return (the image before lessening)
procedure TMainForm.ForthStep(Sender: TObject);
begin
  seC.MaxValue := OrigClNumber;
  seC.Value := OrigClNumber;
  sC := OrigClNumber;
  CopyBitmapToBitmap(iTempForCancel.Picture.Bitmap, iTemp.Picture.Bitmap);
  ProjectImage(Self);
end;

//Draw the bevel around next active button
procedure TMainForm.ActiveButton(Sender: TObject; button: TButton);
begin
  Canvas.Pen.Color := clWhite;
  Canvas.Rectangle(button.Left - 2, button.Top - 2,
    button.Left + button.Width + 2, button.Top + button.Height + 2);
end;

//Delete the bevel around button
procedure TMainForm.InactiveButton(Sender: TObject; button: TButton);
begin
  Canvas.Pen.Color := Color;
  Canvas.Rectangle(button.Left - 2, button.Top - 2,
    button.Left + button.Width + 2, button.Top + button.Height + 2);
end;

//Make some buttons enabled and some disabled
procedure TMainForm.ChangeEnablility(Sender: TObject; b2, b3, b4: boolean; k: byte);
begin
  lArrow12.Enabled := b2;
  bDetect.Enabled := b2;
  lArrow23.Enabled := b3;
  bLessen.Enabled := b3;
  lC.Enabled := b3;
  seC.Enabled := b3;
  if b3 or b4 then
    miSave.Enabled := true
  else
    miSave.Enabled := false;
  tbSave.Enabled := miSave.Enabled;
  lArrow34.Enabled := b4;
  bReturn.Enabled := b4;
  Canvas.Pen.Width := 3;
  InactiveButton(Self, bLoad);
  InactiveButton(Self, bDetect);
  InactiveButton(Self, bLessen);
  InactiveButton(Self, bReturn);
  case k of
    1: ActiveButton(Self, bLoad);
    2: ActiveButton(Self, bDetect);
    3: ActiveButton(Self, bLessen);
    4: ActiveButton(Self, bReturn);
  end;
end;

//Next procedures are understood from their names

procedure TMainForm.FormCreate(Sender: TObject);
var
  Lang, FallbackLang: String;
begin
  iTemp := TImage.Create(nil);
  iTempForCancel := TImage.Create(nil);
  iStart := TImage.Create(nil);
  bmTemp := TBitMap.Create;

  ColorList := TList.Create;

  StatusBar1.Panels[0].Text := Hint1;

  rgDetection.Items.Clear;
  rgDetection.Items.Add(cdItem0);
  rgDetection.Items.Add(cdItem1);
  rgDetection.Items.Add(cdItem2);
  rgDetection.ItemIndex := 0;

  rgSize.Items.Clear;
  rgSize.Items.Add(rgsItem0);
  rgSize.Items.Add(rgsItem1);
  rgSize.Items.Add(rgsItem2);
  rgSize.ItemIndex := 2;

  SchCond := scNone;

  tbOpen.Hint := miOpen.Caption;
  tbSave.Hint := miSave.Caption;
  tbManual.Hint := miManual.Caption;
  tbExit.Hint := miExit.Caption;

  GetLanguageIDs(Lang, FallbackLang);
  HelpPath := 'file://help';
  if FileExists('./help/' + FallbackLang + '/index.html') then
    HelpPath := HelpPath + '/' + FallbackLang
  else
    HelpPath := HelpPath + '/en';
  HTMLHelpDatabase1.BaseURL := HelpPath;
  HomePath := 'http://schememaker.sourceforge.net';
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose := true;
  if (ord(SchCond) > ord(scOpened)) and (not IsSaved) then
    case MessageDlg(QCaption, Question2, mtConfirmation, mbYesNoCancel, 0) of
      mrYes:
        begin
          miSaveClick(Self);
          CanClose := True;
        end;
      mrNo:
        CanClose := True;
      mrCancel:
        CanClose := False;
    end;
end;

procedure TMainForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  iTemp.Destroy;
  iTempForCancel.Destroy;
  iStart.Destroy;
  bmTemp.Destroy;
  ColorList.Destroy;
end;

procedure TMainForm.miOpenClick(Sender: TObject);
var
  let: boolean;
begin
  let := true;
  if ord(SchCond) > ord(scOpened) then
    if MessageDlg(QCaption, Question1, mtConfirmation,
      mbOKCancel, 0) = mrCancel then
      let := false;
  if let then
    if OpenPictureDialog1.Execute then
      begin
        iImage.Picture.LoadFromFile(OpenPictureDialog1.FileName);
        iImage.Width := iImage.Picture.Bitmap.Width;
        iImage.Height := iImage.Picture.Bitmap.Height;
        seW.Value := iImage.Width;
        seH.Value := iImage.Height;
        iScheme.Width := 0;
        iScheme.Height := 0;
        sW := seW.Value;
        sH := seH.Value;
        sP := seP.Value;
        sC := seC.Value;
        bLoad.Enabled := True;
        ChangeEnablility(Self, False, False, False, 1);
        StatusBar1.Panels[0].Text := Hint2;
        SchCond := scOpened;
        isSaved := false;
        pOptions.Visible := true;
        iStart.Picture.LoadFromFile(OpenPictureDialog1.FileName);
        SelectedRect := Rect(0, 0, iImage.Width, iImage.Height);
        DrawSelectedRect(iImage, SelectedRect);
    end;
end;

procedure TMainForm.miSaveClick(Sender: TObject);
begin
  SaveForm.Show;
end;

procedure TMainForm.miExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.miManualClick(Sender: TObject);
begin
  ShowHelpOrErrorForKeyword('', 'HTML/index.html');
end;

procedure TMainForm.miHomeClick(Sender: TObject);
var
  err: string;
begin
  HTMLHelpDatabase1.ShowURL(HomePath, '', err);
end;

procedure TMainForm.miAboutClick(Sender: TObject);
begin
  AboutForm.Show;
end;

procedure TMainForm.seWChange(Sender: TObject);
begin
  case rgSize.ItemIndex of
    0: sW := Round (seW.Value * fseCm.Value);
    1: sW := Round (seW.Value * fseInch.Value);
    2: sW := seW.Value;
  end;
  if cbProportion.Checked then
    begin
      seH.Value := Round(iImage.Height /iImage.Width  * seW.Value);
      sH := Round(iImage.Height /iImage.Width  * sW);
    end;
  if bDetect.Enabled then
    bLoadClick(Self);
end;

procedure TMainForm.seHChange(Sender: TObject);
begin
  case rgSize.ItemIndex of
    0: sH := Round (seH.Value * fseCm.Value);
    1: sH := Round (seH.Value * fseInch.Value);
    2: sH := seH.Value;
  end;
  if cbProportion.Checked then
    begin
      seW.Value := Round(iImage.Width / iImage.Height * seH.Value);
      sW := Round(iImage.Width / iImage.Height * sH);
    end;
  if bDetect.Enabled then
    bLoadClick(Self);
end;

procedure TMainForm.sePChange(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  sP := seP.Value;
  if bDetect.Enabled then
    FirstStep(Self);
  Screen.Cursor := crDefault;
end;

procedure TMainForm.seCChange(Sender: TObject);
begin
  sC := seC.Value;
end;

procedure TMainForm.bLoadClick(Sender: TObject);
var
  let: boolean;
begin
  let := true;
  if ord(SchCond) > ord(scLoaded) then
    if MessageDlg(QCaption, Question1, mtConfirmation,
      mbOKCancel, 0) = mrCancel then
      let := false;
  if let then
    begin
      Screen.Cursor := crHourGlass;
      iTemp.Picture.Assign(iImage.Picture);
      CopyRectOfBitmapToBitmapWithScale(iStart.Picture.Bitmap, iTemp.Picture.Bitmap, 1, 1,
                                        SelectedRect.Left, SelectedRect.Top,
                                        SelectedRect.Right, SelectedRect.Bottom);
      FirstStep(Self);
      ChangeEnablility(Self, True, False, False, 2);
      StatusBar1.Panels[0].Text := Hint3;
      Screen.Cursor := crDefault;
      SchCond := scLoaded;
      isSaved := false;
    end;
end;

procedure TMainForm.bDetectClick(Sender: TObject);
var
  let: boolean;
begin
  let := true;
  if ord(SchCond) > ord(scDetected) then
    if MessageDlg(QCaption, Question1, mtConfirmation,
      mbOKCancel, 0) = mrCancel then
      let := false;
  if let then
    begin
      Screen.Cursor := crHourGlass;
      SecondStep(Self);
      ChangeEnablility(Self, True, True, False, 3);
      StatusBar1.Panels[0].Text := Hint4;
      Screen.Cursor := crDefault;
      SchCond := scDetected;
      isSaved := false;
    end;
end;

procedure TMainForm.bLessenClick(Sender: TObject);
var
  let: boolean;
begin
  let := true;
  if ord(SchCond) > ord(scLessened) then
    if MessageDlg(QCaption, Question1, mtConfirmation,
      mbOKCancel, 0) = mrCancel then
      let := false;
  if let then
    begin
      Screen.Cursor := crHourGlass;
      ThirdStep(Self);
      ChangeEnablility(Self, True, False, True, 4);
      StatusBar1.Panels[0].Text := Hint5;
      Screen.Cursor := crDefault;
      SchCond := scLessened;
      isSaved := false;
    end;
end;

procedure TMainForm.bReturnClick(Sender: TObject);
var
  let: boolean;
begin
  let := true;
  if (ord(SchCond) > ord(scLessened)) and (not IsSaved) then
    if MessageDlg(QCaption, Question1, mtConfirmation,
      mbOKCancel, 0) = mrCancel then
      let := false;
  if let then
    begin
      Screen.Cursor := crHourGlass;
      ForthStep(Self);
      ChangeEnablility(Self, True, True, False, 3);
      StatusBar1.Panels[0].Text := Hint6;
      Screen.Cursor := crDefault;
      SchCond := scDetected;
      isSaved := false;
    end;
end;

procedure TMainForm.cgGrid_ItemClick(Sender: TObject; Index: integer);
begin
  Screen.Cursor := crHourGlass;
  if bDetect.Enabled then
    ProjectImage(Self);
  Screen.Cursor := crDefault;
end;

initialization
  {$I main.lrs}

end.
